;; -*- mode: scheme; coding: utf-8 -*-

;;;; Copyright (C) 2011, 2012
;;;; Free Software Foundation, Inc.

;;;; This library is free software: you can redistribute it and/or
;;;; modify it under the terms of the GNU General Public License as
;;;; published by the Free Software Foundation, either version 3 of
;;;; the License, or (at your option) any later version.

;;;; This library is distributed in the hope that it will be useful, but
;;;; WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;;; General Public License for more details.

;;;; You should have received a copy of the GNU General Public License
;;;; along with this library.  If not, see <http://www.gnu.org/licenses/>.
;;;;

;;; Commentary:

;;; Code:

(define-module (db sqlite)
  ;; guile
  :use-module (ice-9 format)
  :use-module (ice-9 rdelim)
  :use-module (ice-9 popen)

  ;; common
  :use-module (macros reexport)
  :use-module (sqlite3)
  :use-module (db filters)
  :use-module (lists lists)

  :export (sqlite/sqlite-db-file?
	   sqlite/true?
	   sqlite/boolean
	   sqlite/query
	   sqlite/command
	   sqlite/pos
	   sqlite/table-exists?
	   sqlite/build-set-expression
	   sqlite/build-group-by-order-by-expression))

(eval-when (compile load eval)
  (re-export-public-interface (sqlite3)
			      (db filters)))


;;;
;;; API
;;;

(define (sqlite/sqlite-db-file? filename)
  (let* ((port (open-input-pipe (format #f "file ~A" filename)))
	 (str (read-line port)))
    (close-pipe port)
    (string-contains-ci str "sqlite")))

(define (sqlite/true? value)
  (string=? value "t"))

(define (sqlite/boolean value)
  (if value "t" "f"))

(define (sqlite/query db query)
  (let* ((stmt (sqlite-prepare db query))
	 (results (sqlite-fold cons (list) stmt)))
    (sqlite-finalize stmt)
    (reverse! results)))

(define (sqlite/command db command)
  (let ((stmt (sqlite-prepare db command)))
    (sqlite-step stmt)
    (sqlite-finalize stmt)
    (if #f #f)))

(define (sqlite/pos item tuples pred idx)
  (list-pos item tuples (lambda (a b) (pred (vector-ref a idx) b)))) 

(define (sqlite/table-exists-str)
  "select * 
     from sqlite_master
    where name = '~A'")

(define (sqlite/table-exists? db t-name)
  (let ((result (sqlite/query db
			      (format #f "~?" (sqlite/table-exists-str)
				      (list t-name)))))
    (and result
	 (not (null? result))
	 (car result))))

(define (sqlite/build-set-expression items . accessor)
  (let ((result #f)
	(first? #t)
	(acc (if (null? accessor) identity (car accessor))))
    (for-each (lambda (item)
		(if first?
		    (begin
		      (set! result (format #f "(~A" (acc item)))
		      (set! first? #f))
		    (set! result (format #f "~A, ~A" result (acc item)))))
	items)
    (if result
	(format #f "~A)" result)
	#f)))

(define (sqlite/build-group-by-order-by-expression items . accessor)
  (let ((result #f)
	(first? #t)
	(acc (if (null? accessor) identity (car accessor))))
    (for-each (lambda (item)
		(if first?
		    (begin
		      (set! result (format #f "~A" (acc item)))
		      (set! first? #f))
		    (set! result (format #f "~A, ~A" result (acc item)))))
	items)
    (if result
	(format #f "~A" result)
	#f)))


#!

(use-modules (db sqlite))
(reload-module (resolve-module '(db sqlite)))

!#