;; -*- mode: scheme; coding: utf-8 -*-

;;;; Copyright (C) 2011, 2012
;;;; Free Software Foundation, Inc.
;;;;
;;;; This library is free software; you can redistribute it and/or
;;;; modify it under the terms of the GNU General Public License as
;;;; published by the Free Software Foundation; either version 3 of
;;;; the License, or (at your option) any later version.
;;;; 
;;;; This library is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;;; General Public License for more details.
;;;; 
;;;; You should have received a copy of the GNU General Public License
;;;; along with this library; if not, write to the Free Software
;;;; Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
;;;; 02110-1301 USA
;;;;

;;; Commentary:

;;; Code:

(define-module (system locale)
  ;; guile
  :use-module (ice-9 rdelim)
  :use-module (ice-9 popen)
  ;; common
  :use-module (system aglobs)
  :use-module (macros reexport)

  :export (sys/get-locales
	   sys/get-utf8-locales
	   sys/get-idioms
	   sys/get-utf8-idioms
	   sys/get-utf8-idioms-radicals
	   sys/get-utf8-fallback))


(define sys/get-locales #f)
(define sys/get-utf8-locales #f)
(define sys/get-idioms #f)
(define sys/get-utf8-idioms #f)
(define sys/get-utf8-idioms-radicals #f)
(define sys/get-utf8-fallback #f)


(eval-when (compile load eval)
  (re-export-public-interface (system aglobs))

  (define (get-locales)
    (let ((results (list))
	  (s (open-input-pipe "locale -a")))
      (do ((line (read-line s) (read-line s)))
	  ((eof-object? line))
	(set! results (cons line results)))
      (close s)
      (reverse! results)))

  (define (get-utf8-locales locales)
    (let ((valid-utf8-encoding '("utf8" "UTF-8")))
      (filter (lambda (locale)
		(let ((splitted (string-split locale #\.)))
		  (and (not (null? (cdr splitted)))
		       (let ((encoding (cadr splitted)))
			 (or (string=? encoding (car valid-utf8-encoding))
			     (string=? encoding (cadr valid-utf8-encoding)))))))
	      locales)))

  (define (get-idioms locales)
    (let ((idioms (list)))
      (for-each (lambda (locale)
		  (set! idioms (cons (car (string-split locale #\.)) idioms)))
	  locales)
      (reverse! idioms)))

  (define (get-utf8-idioms utf8-locales)
    (let ((results (list)))
      (for-each (lambda (locale)
		  (set! results (cons (car (string-split locale #\.)) results)))
	  utf8-locales)
      (reverse! results)))

  (define (get-utf8-idioms-radicals utf8-idioms)
    (let ((results (list)))
      (for-each (lambda (idiom)
		  (set! results (cons (car (string-split idiom #\_)) results)))
	  utf8-idioms)
      (reverse! results)))

  (define (get-utf8-fallback proposed utf8-idioms-radicals utf8-locales)
    ;; proposed is a list of idioms radicals, i.e., "C", "en"... when
    ;; we find one, by construction the fallback is at the same pos in
    ;; utf8-locales.
    (let* ((its-length (length proposed))
	   (offset (catch 'exit
		     (lambda ()
		       (do ((i 0 (1+ i)))
			   ((= i its-length) #f)
			 (let ((item (list-ref proposed i)))
			   (if (member item utf8-idioms-radicals)
			       (throw 'exit i)))))
		     (lambda (key index) index))))
      (and offset (list-ref utf8-locales offset))))

  (let* ((lang (getenv "LANG"))
	 (locales (get-locales))
	 (utf8-locales (get-utf8-locales locales))
	 (idioms (get-idioms locales))
	 (utf8-idioms (get-utf8-idioms utf8-locales))
	 (utf8-idioms-radicals (get-utf8-idioms-radicals utf8-idioms)))
    (aglobs/set 'lang lang)
    (set! sys/get-locales (lambda () locales))
    (set! sys/get-utf8-locales (lambda () utf8-locales))
    (set! sys/get-idioms (lambda () idioms))
    (set! sys/get-utf8-idioms (lambda () utf8-idioms))
    (set! sys/get-utf8-idioms-radicals (lambda () utf8-idioms-radicals))
    (set! sys/get-utf8-fallback 
	  (lambda (proposed)
	    (get-utf8-fallback proposed utf8-idioms-radicals utf8-locales)))))


#!

(use-modules (system locale))
(reload-module (resolve-module '(system locale)))
,m (system locale)

(aglobs/get 'lang)
(sys/get-locales)
(sys/get-utf8-locales)
(sys/get-idioms)
(sys/get-utf8-idioms)
(sys/get-utf8-idioms-radicals)
(sys/get-utf8-fallback '("C" "en"))

!#
